<?php

namespace HubPublic\Serializers;

use HubPublic\Serializers\Interfaces\SerializerInterface;
use HubPublic\ValueObjects\CustomerInformation;
use HubPublic\Exceptions\BadSerializerValueException;
/**
 * Class CustomerInformationSerializer
 *
 * @package HubPublic\Serializers
 */
class CustomerInformationSerializer implements SerializerInterface
{
    private $keys = ['customerNumber', 'customerFirstName', 'customerLastName', 'customerTitle', 'customerGender', 'customerCompany', 'customerAddress1', 'customerAddress2', 'customerPostalCode', 'customerCity', 'customerState', 'customerCountry', 'customerCountryCode', 'customerDateOfBirth', 'customerPhone', 'customerFax', 'customerEmail', 'billingFirstName', 'billingLastName', 'billingTitle', 'billingGender', 'billingCompany', 'billingAddress1', 'billingAddress2', 'billingPostalCode', 'billingCity', 'billingState', 'billingCountry', 'billingCountryCode', 'billingDateOfBirth', 'billingPhone', 'billingFax', 'billingEmail', 'shippingFirstName', 'shippingLastName', 'shippingTitle', 'shippingGender', 'shippingCompany', 'shippingAddress1', 'shippingAddress2', 'shippingPostalCode', 'shippingCity', 'shippingState', 'shippingCountry', 'shippingCountryCode', 'shippingDateOfBirth', 'shippingPhone', 'shippingFax', 'shippingEmail'];
    /**
     * Deserialize a JSON string.
     *
     * @param string $string JSON string that contains the data.
     *
     * @return \HubPublic\ValueObjects\CustomerInformation New CustomerInformation instance that contains the
     *                                                    deserialized data
     * @throws \HubPublic\Exceptions\BadSerializerValueException If the JSON string is malformed or does not contain
     *                                                          valid data
     */
    public function deserialize($string)
    {
        if (empty($string)) {
            throw new BadSerializerValueException('Given string is empty.');
        }
        $json = json_decode($string, true);
        // error for malformed json strings
        if ($json === null && json_last_error() > 0) {
            throw new BadSerializerValueException('Provided JSON string is malformed and could not be parsed: ' . $string);
        }
        $arguments = $this->_getCustomerInformation($json);
        return new CustomerInformation($arguments['customerNumber'], $arguments['customerFirstName'], $arguments['customerLastName'], $arguments['customerTitle'], $arguments['customerGender'], $arguments['customerCompany'], $arguments['customerAddress1'], $arguments['customerAddress2'], $arguments['customerPostalCode'], $arguments['customerCity'], $arguments['customerState'], $arguments['customerCountry'], $arguments['customerCountryCode'], $arguments['customerDateOfBirth'], $arguments['customerPhone'], $arguments['customerFax'], $arguments['customerEmail'], $arguments['billingFirstName'], $arguments['billingLastName'], $arguments['billingTitle'], $arguments['billingGender'], $arguments['billingCompany'], $arguments['billingAddress1'], $arguments['billingAddress2'], $arguments['billingPostalCode'], $arguments['billingCity'], $arguments['billingState'], $arguments['billingCountry'], $arguments['billingCountryCode'], $arguments['billingDateOfBirth'], $arguments['billingPhone'], $arguments['billingFax'], $arguments['billingEmail'], $arguments['shippingFirstName'], $arguments['shippingLastName'], $arguments['shippingTitle'], $arguments['shippingGender'], $arguments['shippingCompany'], $arguments['shippingAddress1'], $arguments['shippingAddress2'], $arguments['shippingPostalCode'], $arguments['shippingCity'], $arguments['shippingState'], $arguments['shippingCountry'], $arguments['shippingCountryCode'], $arguments['shippingDateOfBirth'], $arguments['shippingPhone'], $arguments['shippingFax'], $arguments['shippingEmail']);
    }
    /**
     * Serialize a value to a JSON string or array.
     *
     * @param CustomerInformation $customerInformation Content to be serialized.
     * @param bool                $encode              Serialize to string?
     *
     * @return array|string Serialized JSON string or array of given content.
     *
     * @throws \HubPublic\Exceptions\BadSerializerValueException If passed argument is not a CustomerInformation.
     */
    public function serialize($customerInformation, $encode = true)
    {
        if (!is_object($customerInformation) || !$customerInformation instanceof CustomerInformation) {
            throw new BadSerializerValueException('Argument is not a CustomerInformation: ' . gettype($customerInformation));
        }
        $json = [];
        foreach ($this->keys as $key) {
            $json[$key] = $customerInformation->{'get' . ucfirst($key)}();
        }
        return $encode ? json_encode($json, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES) : $json;
    }
    /**
     * Check if no customer information is missing and ensure the valid type of every customer information. A customer
     * information array will be returned if validation succeeds.
     *
     * @param array $json Customer information as array
     *
     * @return array Customer information
     */
    private function _getCustomerInformation(array $json)
    {
        $customerInformation = [];
        foreach ($this->keys as $key) {
            $customerInformation[$key] = $json[$key];
        }
        return $customerInformation;
    }
}